<?php

namespace App\domain\travelport;

use Exception;
use GuzzleHttp\Exception\GuzzleException;
use Illuminate\Support\Facades\Storage;

class FlightOfferPricing extends SearchOfferAPI
{
    public function getPrice(string $catalogProductOfferingsIdentifier, array $products)
    {
        $apiUrl = $this->buildApiUrl();

        $payload = $this->createPayload($catalogProductOfferingsIdentifier, $products);

        Storage::put('travelport/api/2-price_request.json', json_encode($payload, JSON_PRETTY_PRINT));

        try {
            $response = $this->client->post($apiUrl, [
                'headers' => $this->getHeaders(),
                'json' => $payload,
            ]);

            Storage::put('travelport/api/2-price_response.json',json_encode(json_decode($response->getBody()), JSON_PRETTY_PRINT));

            return json_decode($response->getBody(), true);
        } catch (Exception $e) {
            echo 'Price Error: ' . $e->getMessage();
            return null;
        } catch (GuzzleException $e) {
            echo 'Error: ' . $e->getMessage();
            return null;
        }
    }

    protected function buildApiUrl(): string
    {
        return "https://{$this->baseURL}/{$this->apiVersion}/air/price/offers/buildfromcatalogproductofferings";
    }

    protected function createPayload(string $catalogProductOfferingsIdentifier,array $products): array
    {
        $productQuery = [];
        foreach ($products as $product){
            $productQuery [] = [
                "CatalogProductOfferingIdentifier" => [
                    "Identifier" => [
                        "value" => $product['offerId']
                    ]
                ],
                "ProductIdentifier" => [
                    [
                        "Identifier" => [
                            "value" => $product['productId']
                        ]
                    ]
                ]
            ];
        }

        return [
            "OfferQueryBuildFromCatalogProductOfferings" => [
                "BuildFromCatalogProductOfferingsRequest" => [
                    "@type" => "BuildFromCatalogProductOfferingsRequestAir",
                    "CatalogProductOfferingsIdentifier" => [
                        "Identifier" => [
                            "value" => $catalogProductOfferingsIdentifier
                        ]
                    ],
                    "CatalogProductOfferingSelection" => [...$productQuery]
                ]
            ]
        ];
    }
}
