<?php

namespace App\domain\travelport\ticketing;

use App\Models\CommitResponse;
use Exception;
use Illuminate\Support\Facades\Log;

class CreateTicketAction
{

    protected string|null $locatorId;
    protected string $reservationId;
    protected string $ticketWorkbenchAuthorityId;

    protected PostCommitInitiateWorkbenchAction $commitInitiateWorkbenchAction;
    protected AddFormOfPaymentAction $addFormOfPaymentAction;
    protected AddPaymentAction $addPaymentAction;
    protected CommitReservationTicketIssuanceAction $commitReservationTicketIssuanceAction;

    public function __construct()
    {
        $this->commitInitiateWorkbenchAction = new PostCommitInitiateWorkbenchAction();
        $this->addFormOfPaymentAction = new AddFormOfPaymentAction();
        $this->addPaymentAction = new AddPaymentAction();
        $this->commitReservationTicketIssuanceAction = new CommitReservationTicketIssuanceAction();
    }


    /**
     * @throws Exception
     */
    public function execute(
        string $reservationId,
        string $payType,
        string $currencyCode,
        float  $amount,
    ): array
    {
        $this->reservationId = $reservationId;

        $commitResponse = $this->getCommitResponse();

        $this->locatorId = $this->getLocatorId($commitResponse);

        if (!$this->locatorId) {
            throw new Exception("Invalid Reservation Id");
        }

        $ticketInitiateResponse = $this->commitInitiateWorkbenchAction->execute($this->locatorId);

        if($ticketInitiateResponse === null) {
            throw new Exception("Ticket work bench not initiated.");
        }

        Log::info("PostCommitInitiateWorkbenchAction Response", $ticketInitiateResponse);

        $this->ticketWorkbenchAuthorityId = $ticketInitiateResponse['ReservationResponse']['Identifier']['value'];

        $offerIdentifier = $ticketInitiateResponse['ReservationResponse']['Reservation']['Offer'][0]['Identifier']['value'];

        $offerId = $ticketInitiateResponse['ReservationResponse']['Reservation']['Offer'][0]['id'];

        $fopResponse = $this->addFormOfPaymentAction->execute($this->ticketWorkbenchAuthorityId);

        $fopIdentifier = $this->ticketWorkbenchAuthorityId;

        $addPaymentResponse = $this->addPaymentAction->execute(
            $this->ticketWorkbenchAuthorityId,
            $payType,
            $currencyCode,
            $amount,
            $fopIdentifier,
            $offerIdentifier,
            $offerId
        );

        $ticketIssuanceResponse = $this->commitReservationTicketIssuanceAction->execute($this->ticketWorkbenchAuthorityId);

        return $ticketIssuanceResponse;
    }

    public function getCommitResponse()
    {
        return CommitResponse::query()
            ->where('reservation_id', $this->reservationId)
            ->first();
    }

    protected function getLocatorId($commitResponse): ?string
    {
        return $commitResponse ? $commitResponse['locator_id'] : null;
    }

}
